"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
// tslint:disable-next-line:ordered-imports
var jsforce = require("jsforce");
var path = require("path");
const checkRetrievalStatus_1 = require("../../../shared/checkRetrievalStatus");
const checkDeploymentStatus_1 = require("../../../shared/checkDeploymentStatus");
const extract_1 = require("../../../shared/extract");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "package_applypatch");
class Applypatch extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        //Connect to the org
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        const apiversion = await conn.retrieveMaxApiVersion();
        let retrieveRequest = {
            apiVersion: apiversion
        };
        //Retrieve Duplicate Rule
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: { name: "StaticResource", members: this.flags.name }
        };
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        //Retrieve Patch
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId);
        if (!metadata_retrieve_result.zipFile)
            throw new core_1.SfdxError("Unable to find the requested Static Resource");
        //adding to temp_sfpowerkit folder
        var zipFileName = "temp_sfpowerkit/unpackaged.zip";
        fs_extra_1.default.mkdirSync("temp_sfpowerkit");
        fs_extra_1.default.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64"
        });
        if (fs_extra_1.default.existsSync(path.resolve(zipFileName))) {
            await extract_1.extract("temp_sfpowerkit");
            fs_extra_1.default.unlinkSync(zipFileName);
            let resultFile = `temp_sfpowerkit/staticresources/${this.flags.name}.resource`;
            if (fs_extra_1.default.existsSync(path.resolve(resultFile))) {
                this.ux.log(`Preparing Patch`);
                fs_extra_1.default.copyFileSync(resultFile, `temp_sfpowerkit/unpackaged.zip`);
                //Deploy patch using mdapi
                conn.metadata.pollTimeout = 300;
                let deployId;
                var zipStream = fs_extra_1.default.createReadStream(zipFileName);
                await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                    if (error) {
                        return console.error(error);
                    }
                    deployId = result;
                });
                this.ux.log(`Deploying Patch with ID  ${deployId.id}`);
                let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
                if (!metadata_deploy_result.success)
                    throw new core_1.SfdxError("Unable to deploy the Patch");
                this.ux.log(`Patch ${this.flags.name} Deployed successfully.`);
                rimraf.sync("temp_sfpowerkit");
                return { status: 1 };
            }
            else {
                this.ux.log(`Patch ${this.flags.name} not found in the org`);
                rimraf.sync("temp_sfpowerkit");
            }
        }
        else {
            this.ux.log(`Patch ${this.flags.name} not found in the org`);
            rimraf.sync("temp_sfpowerkit");
        }
    }
}
Applypatch.description = messages.getMessage("commandDescription");
Applypatch.examples = [
    `$ sfdx sfpowerkit:package:applypatch -n customer_picklist -u sandbox
    Preparing Patch
    Deploying Patch with ID  0Af4Y000003Q7GySAK
    Polling for Deployment Status
    Polling for Deployment Status
    Patch customer_picklist Deployed successfully.
  `
];
Applypatch.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription")
    })
};
// Comment this out if your command does not require an org username
Applypatch.requiresUsername = true;
exports.default = Applypatch;
//# sourceMappingURL=applypatch.js.map