"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = require("fs-extra");
const rimraf = require("rimraf");
const zipDirectory_1 = require("../../../../shared/zipDirectory");
const spawn = require("child-process-promise").spawn;
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_customlabel_create");
class Create extends command_1.SfdxCommand {
    constructor() {
        super(...arguments);
        this.customlabel_language = "en_US";
        this.customlabel_protected = false;
    }
    async run() {
        rimraf.sync("temp_sfpowerkit");
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        // Gives first value in url after https protocol
        const packageName = this.flags.package;
        this.customlabel_fullname = this.flags.ignorepackage
            ? this.flags.fullname
            : `${packageName}_${this.flags.fullname}`;
        this.customlabel_value = this.flags.value;
        this.customlabel_categories = this.flags.categories || null;
        this.customlabel_language =
            this.flags.language || this.customlabel_language;
        this.customlabel_protected =
            this.flags.language || this.customlabel_protected;
        this.customlabel_shortdescription = this.flags.shortdescription;
        var customlabels_metadata = `<?xml version="1.0" encoding="UTF-8"?>
<CustomLabels xmlns="http://soap.sforce.com/2006/04/metadata">
    <labels>
        <fullName>${this.customlabel_fullname}</fullName>${this.customlabel_categories != null
            ? `\n<categories>${this.customlabel_categories}</categories>`
            : ""}
        <shortDescription>${this.customlabel_shortdescription}</shortDescription>
        <language>${this.customlabel_language}</language>
        <protected>${this.customlabel_protected.toString()}</protected>
        <value>${this.customlabel_value}</value>
    </labels>
</CustomLabels>`;
        var package_xml = `<?xml version="1.0" encoding="UTF-8"?>
<Package xmlns="http://soap.sforce.com/2006/04/metadata">
    <types>
        <members>*</members>
        <name>CustomLabel</name>
    </types>
    <version>${this.flags.apiversion}</version>
</Package>`;
        let targetmetadatapath = "temp_sfpowerkit/mdapi/labels/CustomLabels.labels-meta.xml";
        fs.outputFileSync(targetmetadatapath, customlabels_metadata);
        let targetpackagepath = "temp_sfpowerkit/mdapi/package.xml";
        fs.outputFileSync(targetpackagepath, package_xml);
        var zipFile = "temp_sfpowerkit/package.zip";
        await zipDirectory_1.zipDirectory("temp_sfpowerkit/mdapi", zipFile);
        //Deploy Rule
        conn.metadata.pollTimeout = 300;
        let deployId;
        var zipStream = fs.createReadStream(zipFile);
        await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
            if (error) {
                return console.error(error);
            }
            deployId = result;
        });
        if (!this.flags.ignorepackage)
            this.ux.log(`Deployed  Custom Label ${this.customlabel_fullname} in target org with ${this.flags.package}_  prefix, You may now pull and utilize the customlabel:reconcile command `);
        else
            this.ux.log(`Deployed  Custom Label ${this.customlabel_fullname} in target org`);
        rimraf.sync("temp_sfpowerkit");
        return {
            "customlabel.fullname": this.customlabel_fullname
        };
    }
}
Create.description = messages.getMessage("commandDescription");
Create.examples = [
    `$ sfdx sfpowerkit:source:customlabel:create -u fancyScratchOrg1 -n FlashError -v "Memory leaks aren't for the faint hearted" -s "A flashing error --package core"
  Deployed CustomLabel FlashError in target org with core_  prefix, You may now pull and utilize the customlabel:reconcile command
  `
];
Create.flagsConfig = {
    fullname: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("fullnameFlagDescription")
    }),
    value: command_1.flags.string({
        required: true,
        char: "v",
        description: messages.getMessage("valueFlagDescription")
    }),
    categories: command_1.flags.string({
        required: false,
        char: "c",
        description: messages.getMessage("categoriesFlagDescription")
    }),
    language: command_1.flags.string({
        required: false,
        char: "l",
        description: messages.getMessage("languageFlagDescription")
    }),
    protected: command_1.flags.string({
        required: false,
        char: "p",
        description: messages.getMessage("protectedFlagDescription")
    }),
    shortdescription: command_1.flags.string({
        required: true,
        char: "s",
        description: messages.getMessage("shortdescriptionFlagDescription")
    }),
    package: command_1.flags.string({
        required: false,
        description: messages.getMessage("packageFlagDescription")
    }),
    ignorepackage: command_1.flags.boolean({
        char: "i",
        default: false,
        description: messages.getMessage("ignorepackageFlagDescription")
    })
};
// Comment this out if your command does not require an org username
Create.requiresUsername = true;
// Comment this out if your command does not support a hub org username
// protected static supportsDevhubUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Create.requiresProject = true;
exports.default = Create;
//# sourceMappingURL=create.js.map